library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;
use ieee.numeric_std.all;

entity vga is
generic ( 
              H_SIZE : integer := 800;
              V_SIZE : integer := 600
         );
port    (
            reset: in std_logic;
            clk_f: in std_logic;
            hsync: inout std_logic;
            vsync: inout std_logic;
           rgb: out std_logic_vector(2 downto 0)
         );
end vga;

architecture vga_arch of vga is

constant H_PIXELS:   INTEGER:= H_SIZE;
constant H_SYNCTIME: INTEGER:= 80;        
constant H_PORCH:    INTEGER:= 80 ;  
constant H_PERIOD:   INTEGER:= H_PIXELS + H_SYNCTIME  + H_PORCH;

constant V_LINES:    INTEGER:= V_SIZE;
constant V_SYNCTIME: INTEGER:= 1;    
constant V_PORCH:    INTEGER:= 46 ;  
constant V_PERIOD:   INTEGER:= + V_LINES + V_SYNCTIME  + V_PORCH;

signal hcnt: std_logic_vector(10 downto 0);
signal vcnt: std_logic_vector(9  downto 0);

signal pixrg:  std_logic_vector(2 downto 0);
signal blank:  std_logic;
signal pblank: std_logic;

begin 
u1: process(clk_f, reset)
    begin
       if reset = '1' then
           hcnt <= (others => '0');
       elsif (clk_f'event and clk_f = '1') then
             if hcnt < H_PERIOD then
                 hcnt <= hcnt + 1;
             else
                 hcnt <= (others => '0');
             end if;
       end if;
    end process;

u2: process(hsync, reset)
    begin
        if reset='1' then
           vcnt <= (others => '0');
        elsif (hsync'event and hsync = '1') then
            if vcnt < V_PERIOD then
               vcnt <= vcnt + 1;
            else
               vcnt <= (others => '0');
            end if;
        end if;
     end process;

u3: process(clk_f, reset)
    begin
        if reset = '1' then
             hsync <= '0';
        elsif (clk_f'event and clk_f = '1') then
           if (hcnt >= ( H_PIXELS) and hcnt < (H_PIXELS +  H_SYNCTIME)) then
              hsync <= '1';
           else
               hsync <= '0';
           end if;
        end if;
     end process;

u4: process(hsync, reset)
    begin
       if reset = '1' then
            vsync <= '0';
       elsif (hsync'event and hsync = '1') then
           if (vcnt >= (V_LINES ) and vcnt < (V_LINES + V_SYNCTIME)) then
               vsync <= '1';
           else
               vsync <= '0';
           end if;
       end if;
    end process;

u5: blank <= '1' when (hcnt>=H_PIXELS or vcnt>=V_LINES) else '0';

u6: process(clk_f,reset)
    begin
        if reset='1' then
           pblank <= '0';
        elsif rising_edge(clk_f) then
           pblank <= blank;
        end if;
    end process;

u7: process(clk_f,reset)
    begin
       if reset='1' then
            pixrg <= (others => '0');
       elsif (clk_f'event and clk_f='1') then                  
            pixrg <= vcnt(7 downto 5);
            if (unsigned(vcnt) mod 64 = 0) or                          
                  (unsigned(hcnt) mod 64 = 0) then
                      pixrg <= "111"; 
              end if;
         end if;
    end process;

u8: process(clk_f,reset)
    begin
       if reset='1' then
           rgb <= (others => '0');
       elsif (clk_f'event and clk_f='1') then
            
            if pblank='0' then
                case pixrg is
                    when "000" =>   rgb   <=     "100";          -- RED
                    when "001" =>   rgb   <=     "010";          -- GREEN
                    when "010" =>   rgb   <=     "001";          -- BLUE
                    when "011" =>   rgb   <=     "000";          -- BLACK
                    when "100" =>   rgb   <=     "011";          -- CYAN
                    when "101" =>   rgb   <=     "101";          -- PURPLE
                    when "110" =>   rgb   <=     "110";          -- YELLOW
                    when others =>  rgb   <=     "111";          -- WHITE
                end case;
             else
                rgb <=  "000";
            end if;
       end if;
    end process;
end vga_arch;
